% MACHINE Cylinders

:- dec_p_type(cylinders_inv1(sp,rpb)).
cylinders_inv1(PositionsDG,Door_cylinder_locked_p) :-
  pfun(Door_cylinder_locked_p) &
  dompf(Door_cylinder_locked_p,PositionsDG).

:- dec_p_type(cylinders_inv2(sp,rpb)).
cylinders_inv2(PositionsDG,Gear_cylinder_locked_p) :-
  pfun(Gear_cylinder_locked_p) &
  dompf(Gear_cylinder_locked_p,PositionsDG).

:- dec_p_type(cylinders_inv3(sp,rpb,rpb)).
cylinders_inv3(PositionsDG,Door_closed_p,Door_cylinder_locked_p) :-
  foreach(Po in PositionsDG,[M1,M2],
    dec(Po,pos) & dec([M1,M2],bool) &
    (M1 = true implies M2 = true),
    applyTo(Door_cylinder_locked_p,Po,M1) &
    applyTo(Door_closed_p,Po,M2)
  ).

:- dec_p_type(cylinders_inv4(sp,rpb,rpb,rpb)).
cylinders_inv4(PositionsDG,Gear_ext_p,Gear_ret_p,Gear_cylinder_locked_p) :-
  foreach(Po in PositionsDG,[M1,M2,M3],
    dec(Po,pos) & dec([M1,M2,M3],bool) &
    (M1 = true implies (M2 = true or M3 = true)),
    applyTo(Gear_cylinder_locked_p,Po,M1) &
    applyTo(Gear_ext_p,Po,M2) &
    applyTo(Gear_ret_p,Po,M3)
  ).

:- dec_p_type(cylinders_inv5(sp,rpb,rpb)).
cylinders_inv5(PositionsDG,Door_open_p,Gear_cylinder_locked_p) :-
  foreach(Po in PositionsDG,[M],
    dec(Po,pos) & dec(M,bool) &
    (M = false implies Door_open_p = cp(PositionsDG,{true})),
    applyTo(Gear_cylinder_locked_p,Po,M)
  ).

:- dec_p_type(cylinders_init(sp,rpb,rpb,rpb,rpb,rpb,rpb)).
cylinders_init(
    PositionsDG,
    Gear_ext_p,
    Gear_ret_p,
    Door_open_p,
    Door_closed_p,
    Door_cylinder_locked_p, 
    Gear_cylinder_locked_p
    ) :-
  Gear_ext_p = cp(PositionsDG,{true}) &
  Gear_ret_p = cp(PositionsDG,{false}) &
  Door_open_p = cp(PositionsDG,{false}) &
  Door_closed_p = cp(PositionsDG,{true}) &
  Door_cylinder_locked_p = cp(PositionsDG,{true}) &
  Gear_cylinder_locked_p = cp(PositionsDG,{true}).

:- dec_p_type(cylinders_start_GearExtend(sp,rpb,rpb,rpb,rpb)).
cylinders_start_GearExtend(
    PositionsDG,
    Gear_ret_p,
    Door_open_p,
    Gear_cylinder_locked_p,
    Gear_ret_p_
    ) :-
  dec(Po,pos) &
  Po in PositionsDG &
  applyTo(Gear_ret_p,Po,true) &
  Door_open_p = cp(PositionsDG,{true}) &
  applyTo(Gear_cylinder_locked_p,Po,false) &
  foplus(Gear_ret_p,Po,false,Gear_ret_p_).

:- dec_p_type(cylinders_make_GearExtended(sp,rpb,rpb,rpb)).
cylinders_make_GearExtended(
    PositionsDG,
    Gear_ext_p,
    Gear_ret_p,
    Gear_ext_p_
    ) :-
  dec(Po,pos) &
  Po in PositionsDG &
  applyTo(Gear_ext_p,Po,false) &
  applyTo(Gear_ret_p,Po,false) &
  foplus(Gear_ext_p,Po,true,Gear_ext_p_).

:- dec_p_type(cylinders_start_GearRetract(sp,rpb,rpb,rpb,rpb)).
cylinders_start_GearRetract(
    PositionsDG,
    Gear_ext_p,
    Door_open_p,
    Gear_cylinder_locked_p,
    Gear_ext_p_
    ) :-
  dec(Po,pos) &
  Po in PositionsDG &
  applyTo(Gear_ext_p,Po,true) &
  Door_open_p = cp(PositionsDG,{true}) &
  applyTo(Gear_cylinder_locked_p,Po,false) &
  foplus(Gear_ext_p,Po,false,Gear_ext_p_).

:- dec_p_type(cylinders_make_GearRetracted(sp,rpb,rpb,rpb)).
cylinders_make_GearRetracted(
    PositionsDG,
    Gear_ext_p,
    Gear_ret_p,
    Gear_ret_p_
    ) :-
  dec(Po,pos) &
  Po in PositionsDG &
  applyTo(Gear_ext_p,Po,false) &
  applyTo(Gear_ret_p,Po,false) &
  foplus(Gear_ret_p,Po,true,Gear_ret_p_).

:- dec_p_type(cylinders_start_DoorOpening(rpb,rpb,rpb)).
cylinders_start_DoorOpening(
    Door_closed_p,
    Door_cylinder_locked_p,
    Door_closed_p_
    ) :-
  dec(Po,pos) &
  applyTo(Door_closed_p,Po,true) &
  applyTo(Door_cylinder_locked_p,Po,false) &
  foplus(Door_closed_p,Po,false,Door_closed_p_).

:- dec_p_type(cylinders_make_DoorOpen(rpb,rpb,rpb)).
cylinders_make_DoorOpen(
    Door_open_p,
    Door_closed_p,
    Door_open_p_
    ) :-
  dec(Po,pos) &
  applyTo(Door_open_p,Po,false) &
  applyTo(Door_closed_p,Po,false) &
  foplus(Door_open_p,Po,true,Door_open_p_).

:- dec_p_type(cylinders_start_DoorClosing(sp,rpb,rpb,rpb,rpb,rpb)).
cylinders_start_DoorClosing(
    PositionsDG,
    Gear_ext_p,
    Gear_ret_p,
    Door_open_p,
    Gear_cylinder_locked_p,
    Door_open_p_
    ) :-
  dec(Po,pos) &
  Po in PositionsDG &
  applyTo(Door_open_p,Po,true) &
  Gear_ext_p = cp(PositionsDG,{true}) &
  Gear_ret_p = cp(PositionsDG,{true}) &
  Gear_cylinder_locked_p = cp(PositionsDG,{true}) &
  foplus(Door_open_p,Po,false,Door_open_p_).

:- dec_p_type(cylinders_make_DoorClosed(rpb,rpb,rpb)).
cylinders_make_DoorClosed(
    Door_open_p,
    Door_closed_p,
    Door_closed_p_
    ) :-
  dec(Po,pos) &
  applyTo(Door_closed_p,Po,false) &
  applyTo(Door_open_p,Po,false) &
  foplus(Door_closed_p,Po,true,Door_closed_p_).

:- dec_p_type(cylinders_UnlockGearCylinder(sp,rpb,rpb,rpb,rpb,rpb)).
cylinders_UnlockGearCylinder(
    PositionsDG,
    Gear_ext_p,
    Gear_ret_p,
    Door_open_p,
    Gear_cylinder_locked_p,
    Gear_cylinder_locked_p_
    ) :-
  dec(Po,pos) &
  Po in PositionsDG &
  applyTo(Gear_cylinder_locked_p,Po,true) &
  (applyTo(Gear_ext_p,Po,true)
   or
   applyTo(Gear_ret_p,Po,true)
  ) &
  Door_open_p = cp(PositionsDG,{true}) &
  foplus(Gear_cylinder_locked_p,Po,true,Gear_cylinder_locked_p_).

:- dec_p_type(cylinders_LockGearCylinder(sp,rpb,rpb,rpb,rpb)).
cylinders_LockGearCylinder(
    PositionsDG,
    Gear_ext_p,
    Gear_ret_p,
    Gear_cylinder_locked_p,
    Gear_cylinder_locked_p_
    ) :-
  dec(Po,pos) &
  Po in PositionsDG &
  applyTo(Gear_cylinder_locked_p,Po,false) &
  (applyTo(Gear_ext_p,Po,true)
   or
   applyTo(Gear_ret_p,Po,true)
  ) &
  foplus(Gear_cylinder_locked_p,Po,true,Gear_cylinder_locked_p_).

:- dec_p_type(cylinders_UnlockDoorCylinder(sp,rpb,rpb,rpb)).
cylinders_UnlockDoorCylinder(
    PositionsDG,
    Door_closed_p,
    Door_cylinder_locked_p,
    Door_cylinder_locked_p_
    ) :-
  dec(Po,pos) &
  Po in PositionsDG &
  applyTo(Door_closed_p,Po,true) &
  applyTo(Door_cylinder_locked_p,Po,true) &
  foplus(Door_cylinder_locked_p,Po,false,Door_cylinder_locked_p_).

:- dec_p_type(cylinders_LockDoorCylinder(sp,rpb,rpb,rpb)).
cylinders_LockDoorCylinder(
    PositionsDG,
    Door_closed_p,
    Door_cylinder_locked_p,
    Door_cylinder_locked_p_
    ) :-
  dec(Po,pos) &
  Po in PositionsDG &
  applyTo(Door_cylinder_locked_p,Po,false) &
  applyTo(Door_closed_p,Po,true) &
  foplus(Door_cylinder_locked_p,Po,true,Door_cylinder_locked_p_).


